"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const path = require("path");
const chalk_1 = require("chalk");
const bunyan = require("bunyan");
const minimist = require("minimist");
const errors_1 = require("./errors");
const env_1 = require("./lib/env");
const registry_1 = require("./lib/registry");
const index_1 = require("./jobs/clean-up/index");
const index_2 = require("./jobs/fetch-updates/index");
const index_3 = require("./jobs/build-app/index");
const index_4 = require("./jobs/build-android/index");
const registry = new registry_1.JobRegistry();
registry.register('clean-up', index_1.run);
registry.register('fetch-updates', index_2.run);
registry.register('build-app', index_3.run);
registry.register('build-android', index_4.run);
function getJobId() {
    const id = Number(env_1.env.require('IONIC_JOB_ID'));
    if (isNaN(id)) {
        throw new Error('IONIC_JOB_ID invalid');
    }
    return id;
}
async function createJobContext({ name, log, id }) {
    const bindir = env_1.env.require('IONIC_JOBS_BIN');
    const joblog = log.child({ job: name });
    const serverlog = joblog.child({ type: 'server' });
    const userlog = joblog.child({ type: 'user' });
    return {
        id,
        argv: process.argv,
        env: env_1.env,
        c: new chalk_1.default.constructor({ enabled: true }),
        serverlog,
        userlog,
        bindir: path.resolve(bindir),
        codedir: process.cwd(),
    };
}
async function execjob({ name, log, id }) {
    const job = registry.get(name);
    const ctx = await createJobContext({ name, log, id });
    ctx.userlog.info(ctx.c.cyan(`Running Stage ${name} for Job: ${id}`));
    try {
        await job(ctx);
    }
    catch (e) {
        if (e instanceof errors_1.JobException && e.type === 'user') {
            ctx.userlog.error(ctx.c.red(e.msg));
        }
        throw e;
    }
}
async function run() {
    const logfile = env_1.env.get('IONIC_JOBS_LOG', path.resolve(os.homedir(), 'logs', 'jobs.log'));
    const id = getJobId();
    const rootlog = bunyan.createLogger({
        name: 'rootlog',
        type: 'root',
        job_id: id,
        app_id: env_1.env.require('APP_ID'),
        streams: [
            {
                stream: process.stdout,
                level: bunyan.DEBUG,
            },
            {
                type: 'rotating-file',
                path: logfile,
                count: 28,
            },
        ],
    });
    const args = minimist(process.argv.slice(2));
    const jobName = args._[0];
    try {
        if (!jobName) {
            throw new Error(`Must provide job type.`);
        }
        rootlog.info(`Job start: ${jobName}.`);
        await execjob({ name: jobName, log: rootlog, id });
        rootlog.info(`Job succeeded: ${jobName}.`);
    }
    catch (e) {
        rootlog.error(e);
        rootlog.error(`Job failed: ${jobName}.`);
        process.exitCode = 1;
    }
}
exports.run = run;
