"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const ECR = require("aws-sdk/clients/ecr");
const S3 = require("aws-sdk/clients/s3");
const errors_1 = require("../../errors");
const api_1 = require("../../lib/api");
const aws_1 = require("../../lib/aws");
const cordova_1 = require("../../lib/cordova");
const docker_1 = require("../../lib/docker");
const project_1 = require("../../lib/project");
const fs_1 = require("../../lib/fs");
const shell_1 = require("../../lib/shell");
const git_1 = require("../../lib/git");
async function run(ctx) {
    const ecr = new aws_1.ECRClient(new ECR({
        apiVersion: '2015-09-21',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
        region: ctx.env.require('AWS_REGION'),
    }));
    const s3 = new aws_1.S3Client(new S3({
        apiVersion: '2006-03-01',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
    }));
    await prepareDocker(ctx, ecr);
    await prepareApp(ctx, s3);
    await buildAndroidApp(ctx);
    await uploadArtifact(ctx, s3);
}
exports.run = run;
async function prepareDocker(ctx, ecr) {
    const environment = ctx.env.require('ENVIRONMENT');
    const dockerImage = ctx.env.require('PACKAGE_ANDROID_IMAGE');
    const dir = path.dirname(path.dirname(ctx.bindir));
    const token = await ecr.getLoginCredentials();
    await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['login', '--username', token.username, '--password', token.password, token.endpoint], {});
    try {
        if (environment === 'DEVELOPMENT') {
            ctx.serverlog.info(`Building ${dockerImage}...`);
            const dockerDir = path.resolve(dir, 'docker', 'package-android');
            await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['build', '-t', dockerImage, dockerDir], {});
        }
        else {
            ctx.serverlog.info(`Pulling ${dockerImage}...`);
            await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['pull', dockerImage], {});
        }
    }
    catch (e) {
        ctx.serverlog.error(e);
    }
}
exports.prepareDocker = prepareDocker;
async function prepareApp(ctx, s3) {
    const dockerImage = ctx.env.require('PACKAGE_ANDROID_IMAGE');
    const snapshotBucket = ctx.env.require('SNAPSHOT_S3_BUCKET');
    const snapshotId = ctx.env.require('SNAPSHOT_ID');
    try {
        await git_1.writeGitCredentials(ctx);
    }
    catch (e) {
        ctx.serverlog.error(e);
    }
    const dockerEnv = {
        APP_ID: ctx.env.require('APP_ID'),
        CI: ctx.env.require('CI'),
        IONIC_CONFIG_DIRECTORY: `${ctx.codedir}/.ionic`
    };
    ctx.userlog.info(ctx.c.magenta('Parsing ionic.config.json for project settings'));
    const project = await project_1.readProjectFile(ctx);
    ctx.userlog.info(ctx.c.magenta('Project Settings:'));
    ctx.userlog.info(ctx.c.magenta(`\tName: ${project.name}`));
    ctx.userlog.info(ctx.c.magenta(`\tApp ID: ${project.app_id}`));
    ctx.userlog.info(ctx.c.magenta(`\tType: ${project.type}`));
    ctx.userlog.info(ctx.c.magenta('Printing Build Environment Details'));
    await docker_1.prettyUserCommand(ctx, dockerImage, ['ionic', 'info'], dockerEnv);
    await docker_1.prettyUserCommand(ctx, dockerImage, ['npm', 'install', '--quiet', '--no-optional']);
    const filename = `${snapshotId}.zip`;
    const wwwpath = path.resolve(ctx.codedir, 'www');
    const zippath = path.resolve(ctx.codedir, filename);
    try {
        await fs_1.fsMkdir(wwwpath);
    }
    catch (e) {
        if (e.code !== 'EEXIST') {
            ctx.serverlog.error(e);
        }
    }
    const ws = fs.createWriteStream(zippath);
    await s3.download(snapshotBucket, filename, ws);
    await shell_1.runcmd({ outlog: ctx.serverlog }, 'unzip', ['-qo', zippath], { cwd: wwwpath });
    await fs_1.fsUnlink(zippath);
}
exports.prepareApp = prepareApp;
async function buildAndroidApp(ctx) {
    const dockerImage = ctx.env.require('PACKAGE_ANDROID_IMAGE');
    const appId = ctx.env.require('APP_ID');
    const profileTag = ctx.env.get('CREDENTIAL_PROFILE_TAG');
    let credentials;
    if (profileTag) {
        credentials = (await api_1.getAndroidCredentials(ctx, appId, profileTag)).data;
    }
    await cordova_1.cordovaPlatformAdd(ctx, dockerImage, 'android');
    await cordova_1.cordovaBuildAndroid(ctx, dockerImage, { credentials });
}
exports.buildAndroidApp = buildAndroidApp;
async function uploadArtifact(ctx, s3) {
    const packageBucket = ctx.env.require('PACKAGE_S3_BUCKET');
    const snapshotId = ctx.env.require('SNAPSHOT_ID');
    const buildType = ctx.env.get('BUILD_TYPE', 'debug');
    const appName = ctx.env.require('APP_NAME');
    const branch = ctx.env.require('BRANCH');
    const sha = ctx.env.require('COMMIT_SHA');
    const apkNameCordova7 = `android-${buildType}.apk`;
    const apkNameCordova8 = `app-${buildType}.apk`;
    const apkPathCordova7 = 'platforms/android/build/outputs/apk';
    const apkPathCordova8 = `platforms/android/app/build/outputs/apk/${buildType}`;
    let artifactPath;
    try {
        [artifactPath] = await fs_1.findFiles(apkPathCordova7, apkNameCordova7);
    }
    catch (e) {
        [artifactPath] = await fs_1.findFiles(apkPathCordova8, apkNameCordova8);
    }
    if (!artifactPath) {
        const msg = (`${ctx.c.red(`Could not find ${apkNameCordova7} in ${apkPathCordova7} or ${apkNameCordova8} in ${apkPathCordova8}`)}\n` +
            `${ctx.c.yellow(`This may be caused by using Crosswalk, which is not supported by Ionic Pro.`)}`);
        throw new errors_1.JobException(msg, 'user');
    }
    const artifact = fs.createReadStream(artifactPath);
    const key = `builds/${snapshotId}-${buildType}.apk`;
    const contentDisposition = `attachment; filename=${appName}-${branch}-${sha}-${buildType}.apk`;
    ctx.serverlog.info({ bucket: packageBucket, key }, `Uploading ${artifactPath} to s3://${packageBucket}/${key}`);
    ctx.userlog.info(ctx.c.yellow('Uploading APK...'));
    await s3.upload(packageBucket, key, artifact, { ContentDisposition: contentDisposition });
    ctx.userlog.info(ctx.c.green('APK uploaded successfully!'));
}
exports.uploadArtifact = uploadArtifact;
