"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const archiver = require("archiver");
const ECR = require("aws-sdk/clients/ecr");
const S3 = require("aws-sdk/clients/s3");
const errors_1 = require("../../errors");
const aws_1 = require("../../lib/aws");
const docker_1 = require("../../lib/docker");
const project_1 = require("../../lib/project");
const npm_1 = require("../../lib/npm");
const shell_1 = require("../../lib/shell");
const fs_1 = require("../../lib/fs");
const app_analytics_1 = require("../../lib/app-analytics");
const git_1 = require("../../lib/git");
async function run(ctx) {
    const ecr = new aws_1.ECRClient(new ECR({
        apiVersion: '2015-09-21',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
        region: ctx.env.require('AWS_REGION'),
    }));
    const s3 = new aws_1.S3Client(new S3({
        apiVersion: '2006-03-01',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
    }));
    await prepareDocker(ctx, ecr);
    await buildApp(ctx);
    await uploadApp(ctx, s3);
    await legacyUploadApp(ctx, s3);
}
exports.run = run;
async function prepareDocker(ctx, ecr) {
    const environment = ctx.env.require('ENVIRONMENT');
    const dockerImage = ctx.env.require('SNAPSHOT_IMAGE');
    const dir = path.dirname(path.dirname(ctx.bindir));
    const token = await ecr.getLoginCredentials();
    await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['login', '--username', token.username, '--password', token.password, token.endpoint], {});
    try {
        if (environment === 'DEVELOPMENT') {
            ctx.serverlog.info(`Building ${dockerImage}...`);
            const dockerDir = path.resolve(dir, 'docker', 'snapshot');
            await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['build', '-t', dockerImage, dockerDir], {});
        }
        else {
            ctx.serverlog.info(`Pulling ${dockerImage}...`);
            await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['pull', dockerImage], {});
        }
    }
    catch (e) {
        ctx.serverlog.error(e);
    }
}
exports.prepareDocker = prepareDocker;
async function buildApp(ctx) {
    const image = ctx.env.require('SNAPSHOT_IMAGE');
    const packageJson = await npm_1.readPackageJson(ctx.codedir, 'user');
    const dockerEnv = {
        APP_ID: ctx.env.require('APP_ID'),
        CI: ctx.env.require('CI'),
        SNAPSHOT_ID: ctx.env.require('SNAPSHOT_ID'),
        IONIC_CONFIG_DIRECTORY: `${ctx.codedir}/.ionic`
    };
    ctx.userlog.info(ctx.c.magenta('Parsing ionic.config.json for project settings'));
    const project = await project_1.readProjectFile(ctx);
    ctx.userlog.info(ctx.c.magenta('Project Settings:'));
    ctx.userlog.info(ctx.c.magenta(`\tName: ${project.name}`));
    ctx.userlog.info(ctx.c.magenta(`\tApp ID: ${project.app_id}`));
    ctx.userlog.info(ctx.c.magenta(`\tType: ${project.type}`));
    ctx.userlog.info(ctx.c.magenta('Printing Build Environment Details'));
    await docker_1.prettyUserCommand(ctx, image, ['ionic', 'info'], dockerEnv);
    try {
        await app_analytics_1.scrapeApp(project, packageJson, ctx.codedir);
    }
    catch (e) {
        ctx.serverlog.error(e);
    }
    try {
        await git_1.writeGitCredentials(ctx);
    }
    catch (e) {
        ctx.serverlog.error(e);
    }
    if (packageJson.scripts && packageJson.scripts.build) {
        // Build app
        ctx.userlog.info(ctx.c.yellow('Build script detected in package.json...'));
        await docker_1.prettyUserCommand(ctx, image, ['npm', 'install', '--quiet', '--no-optional'], dockerEnv);
        await docker_1.prettyUserCommand(ctx, image, ['npm', 'run', 'build'], dockerEnv);
    }
    else {
        ctx.userlog.info(ctx.c.yellow('No build script detected in package.json. Skipping install and build steps.'));
    }
    // Generate app manifest
    ctx.userlog.info(ctx.c.yellow('Generating app manifest...'));
    await docker_1.prettyUserCommand(ctx, image, ['ionic', 'deploy', 'manifest'], dockerEnv);
}
exports.buildApp = buildApp;
async function legacyUploadApp(ctx, s3) {
    const wwwpath = path.resolve(ctx.codedir, 'www');
    try {
        await fs_1.fsAccess(wwwpath);
    }
    catch (e) {
        throw new errors_1.JobException('No www directory found. ' +
            'You must either have a build script which produces ' +
            'your app in the www directory or commit the ' +
            'www directory to version control.', 'user');
    }
    const bucket = ctx.env.require('SNAPSHOT_S3_BUCKET');
    const snapshotId = ctx.env.require('SNAPSHOT_ID');
    ctx.userlog.info(ctx.c.yellow('Uploading build...'));
    try {
        const archive = archiver('zip');
        archive.directory(wwwpath, '');
        archive.finalize();
        await s3.upload(bucket, `${snapshotId}.zip`, archive);
        ctx.userlog.info(ctx.c.green('Build uploaded successfully!'));
    }
    catch (e) {
        ctx.userlog.error(ctx.c.red('Failed to upload build to storage. Please retry your build.'));
        throw e;
    }
}
exports.legacyUploadApp = legacyUploadApp;
async function uploadApp(ctx, s3) {
    const bucket = ctx.env.require('PRO_DEPLOY_S3_BUCKET');
    const appId = ctx.env.require('APP_ID');
    const snapshotId = ctx.env.require('SNAPSHOT_ID');
    const wwwpath = path.resolve(ctx.codedir, 'www');
    const f = await fs_1.fsReadFile(path.resolve(wwwpath, 'pro-manifest.json'), { encoding: 'utf8' });
    ctx.userlog.info(ctx.c.yellow('Uploading app files...'));
    JSON.parse(f).forEach(async (fileObject) => {
        try {
            ctx.serverlog.debug(`Uploading ${fileObject.file}...`);
            let newKey = fs.createReadStream(path.resolve(wwwpath, fileObject.href));
            await s3.upload(bucket, `${appId}/${snapshotId}/${fileObject.href}`, newKey);
        }
        catch (e) {
            ctx.userlog.error(ctx.c.red('Failed uploading from manifest, please retry your build.'));
            throw e;
        }
    });
    ctx.userlog.info(ctx.c.yellow(`Uploading manifest...`));
    await s3.upload(bucket, `${appId}/${snapshotId}/pro-manifest.json`, fs.createReadStream(path.resolve(wwwpath, 'pro-manifest.json')));
    ctx.userlog.info(ctx.c.green('Manifest uploaded successfully!'));
}
exports.uploadApp = uploadApp;
