"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const env_1 = require("./env");
const fs_1 = require("./fs");
const util_1 = require("util");
const apple = require('app-store-scraper');
const google = require('google-play-scraper');
const { parseString } = require('xml2js');
const parseXml = util_1.promisify(parseString);
const Analytics = require('analytics-node');
async function scrapeApp(ionicConfig, packageJson, codeDir) {
    const segmentKey = env_1.env.require('SEGMENT_WRITE_KEY');
    const userId = env_1.env.require('IONIC_USER_ID');
    const appId = env_1.env.require('APP_ID');
    const analytics = new Analytics(segmentKey);
    const flush = util_1.promisify(analytics.flush.bind(analytics));
    const configXml = await readXml(`${codeDir}/config.xml`);
    const appData = { appId: appId };
    appData.ionic_type = ionicConfig.type;
    const deps = packageJson.dependencies;
    if (deps) {
        appData.pro_client_version = hardVersion(deps['@ionic/pro']);
        appData.ionic_angular_version = hardVersion(deps['ionic-angular']);
        appData.cordova_ionic_version = hardVersion(deps['cordova-plugin-ionic']);
    }
    let appStoreInfo;
    let playStoreInfo;
    if (configXml && configXml.widget) {
        if (configXml.widget.$ && configXml.widget.$) {
            appStoreInfo = await scrapeApple(configXml.widget.$);
            Object.assign(appData, appStoreInfo);
            playStoreInfo = await scrapeGoogle(configXml.widget.$);
            Object.assign(appData, playStoreInfo);
        }
        if (configXml.widget.plugin) {
            appData.plugins = configXml.widget.plugin.map((x) => x.$.name).join();
        }
    }
    analytics.track({
        userId: userId,
        event: 'Ionic App Info',
        properties: appData
    });
    await flush();
}
exports.scrapeApp = scrapeApp;
function hardVersion(semver) {
    if (semver) {
        return semver.split('.').map((s) => s.replace(/\D/g, '')).join('.');
    }
}
function processApple(obj) {
    let info = { ios_store: false };
    if (!obj) {
        return info;
    }
    info.ios_store = true;
    info.ios_id = obj.id;
    info.ios_bundle_id = obj.appId;
    info.ios_title = obj.title;
    info.ios_url = obj.url;
    info.ios_description = obj.description;
    info.ios_genre = obj.primaryGenre;
    info.ios_required_os_version = obj.requiredOsVersion;
    info.ios_released = new Date(obj.released);
    info.ios_updated = new Date(obj.updated);
    info.ios_version = obj.version;
    info.ios_free = obj.free;
    info.ios_developer_website = obj.developerWebsite;
    info.ios_developer_url = obj.developerUrl;
    return info;
}
function processGoogle(obj) {
    let info = { play_store: false };
    if (!obj) {
        return info;
    }
    info.play_store = true;
    info.play_title = obj.title;
    info.play_url = obj.url;
    info.play_bundle_id = obj.appId;
    info.play_free = obj.free;
    info.play_summary = obj.summary;
    info.play_min_installs = obj.minInstalls;
    info.play_max_installs = obj.maxInstalls;
    info.play_score = obj.score;
    info.play_developer_website = obj.developerWebsite;
    info.play_developer_email = obj.developerEmail;
    info.play_developer = obj.developer;
    info.play_updated = new Date(obj.updated);
    info.play_version = obj.version;
    info.play_genre = obj.genre;
    info.play_required_android_version = obj.androidVersion;
    info.play_description = obj.description;
    return info;
}
async function scrapeApple(widget) {
    let bundleId = widget['ios-CFBundleIdentifier'] || widget['id'];
    if (!bundleId || bundleId === 'io.ionic.starter')
        return processApple();
    try {
        let appleInfo = await apple.app({ appId: bundleId });
        return processApple(appleInfo);
    }
    catch (e) {
        return processApple();
    }
}
async function scrapeGoogle(widget) {
    let bundleId = widget['android-packageName'] || widget['id'];
    if (!bundleId || bundleId === 'io.ionic.starter')
        return processApple();
    if (!bundleId)
        return processGoogle();
    try {
        let googleInfo = await google.app({ appId: bundleId });
        return processGoogle(googleInfo);
    }
    catch (e) {
        return processGoogle();
    }
}
async function readXml(path) {
    try {
        const f = await fs_1.fsReadFile(path, { encoding: 'utf-8' });
        const o = await parseXml(f);
        return o;
    }
    catch (e) {
        if (e instanceof SyntaxError) {
            console.log(`Invalid XML in ${path}`);
            return;
        }
    }
}
